@@ -60,7 +60,6 @@
 struct _CallsApplication {
   AdwApplication      parent_instance;
 
-  gboolean            daemon;
   CallsRinger        *ringer;
   CallsNotifier      *notifier;
   CallsRecordStore   *record_store;
@@ -98,8 +97,14 @@ quit_calls (CallsApplication *self)
   if (self->shutdown)
     return;
 
-  gtk_application_remove_window (GTK_APPLICATION (self), GTK_WINDOW (self->main_window));
-  gtk_application_remove_window (GTK_APPLICATION (self), GTK_WINDOW (self->call_window));
+  if (self->main_window)
+    gtk_application_remove_window (GTK_APPLICATION (self), GTK_WINDOW (self->main_window));
+
+  if (self->call_window)
+    gtk_application_remove_window (GTK_APPLICATION (self), GTK_WINDOW (self->call_window));
+
+  if (g_application_get_flags (G_APPLICATION (self)) & G_APPLICATION_IS_SERVICE)
+    g_application_release (G_APPLICATION (self));
 
   self->shutdown = TRUE;
 }
@@ -238,21 +243,6 @@ set_default_plugins_action (GSimpleAction *action,
 }
 
 
-static void
-set_daemon_action (GSimpleAction *action,
-                   GVariant      *parameter,
-                   gpointer       user_data)
-{
-  CallsApplication *self = CALLS_APPLICATION (user_data);
-  gboolean daemon = g_variant_get_boolean (parameter);
-
-  self->daemon = daemon;
-
-  g_debug ("Application %smarked as daemon",
-           daemon ? "" : "not ");
-}
-
-
 #define DIALLING    "0-9*#+ABCD"
 #define SIGNALLING  ",TP!W@X"
 #define VISUAL      "[:space:]\\-.()t/"
@@ -444,7 +434,6 @@ static const GActionEntry actions[] =
 {
   { "set-plugin-names", set_plugin_names_action, "as" },
   { "set-default-plugins", set_default_plugins_action, NULL },
-  { "set-daemon", set_daemon_action, "b" },
   { "dial", dial_action, "s" },
   { "copy-number", copy_number, "s" },
   /* TODO About dialog { "about", show_about, NULL}, */
@@ -479,6 +468,7 @@ static void
 startup (GApplication *application)
 {
   g_autoptr (GError) error = NULL;
+  CallsApplication *self = CALLS_APPLICATION (application);
 
   G_APPLICATION_CLASS (calls_application_parent_class)->startup (application);
 
@@ -509,6 +499,16 @@ startup (GApplication *application)
                            application,
                            G_CONNECT_SWAPPED);
 
+  if (g_application_get_flags (application) & G_APPLICATION_IS_SERVICE) {
+    g_application_hold (application);
+    g_debug ("Enable daemon mode");
+  }
+
+  start_proper (self);
+  g_action_group_activate_action (G_ACTION_GROUP (application),
+                                  "set-default-plugins",
+                                  NULL);
+
   manager_state_changed_cb (application);
 }
 
@@ -546,16 +546,8 @@ calls_application_command_line (GApplication            *application,
     g_action_group_activate_action (G_ACTION_GROUP (application),
                                     "set-plugin-names",
                                     plugins);
-  } else {
-    g_action_group_activate_action (G_ACTION_GROUP (application),
-                                    "set-default-plugins",
-                                    NULL);
   }
 
-  g_action_group_activate_action (G_ACTION_GROUP (application),
-                                  "set-daemon",
-                                  g_variant_new_boolean (g_variant_dict_contains (options, "daemon")));
-
   if (g_variant_dict_lookup (options, "dial", "&s", &arg))
     g_action_group_activate_action (G_ACTION_GROUP (application),
                                     "dial", g_variant_new_string (arg));
@@ -592,6 +584,13 @@ app_shutdown (GApplication *application)
     g_main_context_iteration (context, TRUE);
   }
 
+  while (self->record_store && calls_record_store_is_busy (self->record_store)) {
+    if (g_timer_elapsed (timer, NULL) > 10)
+      break;
+
+    g_main_context_iteration (context, TRUE);
+  }
+
   cui_uninit ();
 
   G_APPLICATION_CLASS (calls_application_parent_class)->shutdown (application);
@@ -697,16 +696,12 @@ static void
 activate (GApplication *application)
 {
   CallsApplication *self = CALLS_APPLICATION (application);
-  gboolean present;
 
   g_debug ("Activated");
 
   start_proper (self);
-  present = !self->daemon;
 
-  if (present || self->uri) {
-    gtk_window_present (GTK_WINDOW (self->main_window));
-  }
+  gtk_window_present (GTK_WINDOW (self->main_window));
 
   if (self->uri) {
     if (g_str_has_prefix (self->uri, "tel:"))
@@ -826,12 +821,6 @@ calls_application_init (CallsApplication *self)
       _("The name of the plugins to load"),
       _("PLUGIN")
     },
-    {
-      "daemon", 'd', G_OPTION_FLAG_NONE,
-      G_OPTION_ARG_NONE, NULL,
-      _("Whether to present the main window on startup"),
-      NULL
-    },
     {
       "dial", 'l', G_OPTION_FLAG_NONE,
       G_OPTION_ARG_STRING, NULL,
