@@ -24,7 +24,7 @@
 
 #define G_LOG_DOMAIN "CallsRecordStore"
 
-#include "config.h"
+#include "calls-config.h"
 
 #include "calls-call-record.h"
 #include "calls-manager.h"
@@ -41,9 +41,13 @@
 #define RECORD_STORE_FILENAME "records.db"
 #define RECORD_STORE_VERSION  2
 
+enum {
+  SIGNAL_DB_DONE,
+  N_SIGNALS
+};
+static guint signals[N_SIGNALS];
 
-typedef enum
-{
+typedef enum {
   STARTED,
   ANSWERED,
   ENDED
@@ -53,12 +57,9 @@ typedef enum
 static CallsCallRecordState
 state_to_record_state (CuiCallState call_state)
 {
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
   switch (call_state) {
   case CUI_CALL_STATE_CALLING:
-  case CUI_CALL_STATE_ALERTING:
   case CUI_CALL_STATE_INCOMING:
-  case CUI_CALL_STATE_WAITING:
     return STARTED;
 
   case CUI_CALL_STATE_ACTIVE:
@@ -71,20 +72,37 @@ state_to_record_state (CuiCallState call_state)
   default:
     g_assert_not_reached ();
   }
-#pragma GCC diagnostic warning "-Wdeprecated-declarations"
 }
 
 
-struct _CallsRecordStore
+static const char *
+record_state_to_string (CallsCallRecordState state)
 {
-  GtkApplicationWindow parent_instance;
+  switch (state) {
+  case STARTED:
+    return "started";
+  case ANSWERED:
+    return "answered";
+  case ENDED:
+    return "ended";
+  default:
+    return "unknown";
+  }
+}
 
-  gchar *filename;
-  GomAdapter *adapter;
-  GomRepository *repository;
+
+struct _CallsRecordStore {
+  GObject              parent_instance;
+
+  gchar               *filename;
+  GomAdapter          *adapter;
+  GomRepository       *repository;
+  int                  busy;
+
+  GListStore          *list_store;
 };
 
-G_DEFINE_TYPE (CallsRecordStore, calls_record_store, G_TYPE_LIST_STORE);
+G_DEFINE_TYPE (CallsRecordStore, calls_record_store, G_TYPE_OBJECT);
 
 
 static void
@@ -124,8 +142,8 @@ delete_record_cb (GomResource      *resource,
 
 
 static void
-delete_call_cb (CallsCallRecord    *record,
-                CallsRecordStore   *self)
+delete_call_cb (CallsCallRecord  *record,
+                CallsRecordStore *self)
 {
   gom_resource_delete_async (GOM_RESOURCE (record),
                              (GAsyncReadyCallback) delete_record_cb,
@@ -138,8 +156,8 @@ load_calls_fetch_cb (GomResourceGroup *group,
                      GAsyncResult     *res,
                      CallsRecordStore *self)
 {
-  gboolean ok;
   g_autoptr (GError) error = NULL;
+  gboolean ok;
   guint count, i;
   gpointer *records;
 
@@ -149,6 +167,7 @@ load_calls_fetch_cb (GomResourceGroup *group,
   if (error) {
     g_debug ("Error fetching call records: %s",
              error->message);
+    goto exit;
     return;
   }
   g_assert (ok);
@@ -175,7 +194,7 @@ load_calls_fetch_cb (GomResourceGroup *group,
                       self);
   }
 
-  g_list_store_splice (G_LIST_STORE (self),
+  g_list_store_splice (G_LIST_STORE (self->list_store),
                        0,
                        0,
                        records,
@@ -183,6 +202,9 @@ load_calls_fetch_cb (GomResourceGroup *group,
 
   g_free (records);
   g_object_unref (group);
+exit:
+  self->busy--;
+  g_object_unref (self);
 }
 
 
@@ -191,8 +213,8 @@ load_calls_find_cb (GomRepository    *repository,
                     GAsyncResult     *res,
                     CallsRecordStore *self)
 {
-  GomResourceGroup *group;
   g_autoptr (GError) error = NULL;
+  GomResourceGroup *group;
   guint count;
 
   group = gom_repository_find_finish (repository,
@@ -201,7 +223,7 @@ load_calls_find_cb (GomRepository    *repository,
   if (error) {
     g_debug ("Error finding call records in database `%s': %s",
              self->filename, error->message);
-    return;
+    goto exit;
   }
   g_assert (group != NULL);
 
@@ -209,17 +231,20 @@ load_calls_find_cb (GomRepository    *repository,
   if (count == 0) {
     g_debug ("No call records found in database `%s'",
              self->filename);
-    return;
+    goto exit;
   }
 
   g_debug ("Found %u call records in database `%s', fetching",
            count, self->filename);
-  gom_resource_group_fetch_async
-    (group,
-     0,
-     count,
-     (GAsyncReadyCallback)load_calls_fetch_cb,
-     self);
+  self->busy++;
+  gom_resource_group_fetch_async (group,
+                                  0,
+                                  count,
+                                  (GAsyncReadyCallback) load_calls_fetch_cb,
+                                  g_object_ref (self));
+exit:
+  self->busy--;
+  g_object_unref (self);
 }
 
 
@@ -230,7 +255,7 @@ load_calls (CallsRecordStore *self)
   GomSorting *sorting;
 
   filter = gom_filter_new_is_not_null
-    (CALLS_TYPE_CALL_RECORD, "start");
+             (CALLS_TYPE_CALL_RECORD, "start");
 
   sorting = gom_sorting_new (CALLS_TYPE_CALL_RECORD,
                              "start",
@@ -239,20 +264,21 @@ load_calls (CallsRecordStore *self)
 
   g_debug ("Finding records in call record database `%s'",
            self->filename);
+  self->busy++;
   gom_repository_find_sorted_async (self->repository,
                                     CALLS_TYPE_CALL_RECORD,
                                     filter,
                                     sorting,
-                                    (GAsyncReadyCallback)load_calls_find_cb,
-                                    self);
+                                    (GAsyncReadyCallback) load_calls_find_cb,
+                                    g_object_ref (self));
 
   g_object_unref (G_OBJECT (filter));
 }
 
 
 static void
-set_up_repo_migrate_cb (GomRepository *repo,
-                        GAsyncResult *res,
+set_up_repo_migrate_cb (GomRepository    *repo,
+                        GAsyncResult     *res,
                         CallsRecordStore *self)
 {
   g_autoptr (GError) error = NULL;
@@ -260,21 +286,24 @@ set_up_repo_migrate_cb (GomRepository *repo,
 
   ok = gom_repository_automatic_migrate_finish (repo, res, &error);
   if (!ok) {
-      if (error)
-        g_warning ("Error migrating call record database `%s': %s",
-                   self->filename, error->message);
-      else
-        g_warning ("Unknown error migrating call record database `%s'",
-                   self->filename);
+    if (error)
+      g_warning ("Error migrating call record database `%s': %s",
+                 self->filename, error->message);
+    else
+      g_warning ("Unknown error migrating call record database `%s'",
+                 self->filename);
 
-      g_clear_object (&self->repository);
-      g_clear_object (&self->adapter);
-
-    } else {
+    g_clear_object (&self->repository);
+    g_clear_object (&self->adapter);
+  } else {
     g_debug ("Successfully migrated call record database `%s'",
              self->filename);
     load_calls (self);
   }
+
+  g_signal_emit (self, signals[SIGNAL_DB_DONE], 0, ok);
+  self->busy--;
+  g_object_unref (self);
 }
 
 
@@ -296,13 +325,14 @@ set_up_repo (CallsRecordStore *self)
   g_debug ("Attempting migration of call"
            " record database `%s'",
            self->filename);
-  types = g_list_append (types, (gpointer)CALLS_TYPE_CALL_RECORD);
+  types = g_list_append (types, (gpointer) CALLS_TYPE_CALL_RECORD);
+  self->busy++;
   gom_repository_automatic_migrate_async
     (repo,
-     RECORD_STORE_VERSION,
-     types,
-     (GAsyncReadyCallback)set_up_repo_migrate_cb,
-     self);
+    RECORD_STORE_VERSION,
+    types,
+    (GAsyncReadyCallback) set_up_repo_migrate_cb,
+    g_object_ref (self));
 
   self->repository = repo;
 }
@@ -314,28 +344,27 @@ close_adapter (CallsRecordStore *self)
   g_autoptr (GError) error = NULL;
   gboolean ok;
 
-  if (!self->adapter)
-    {
-      return;
-    }
+  if (!self->adapter) {
+    return;
+  }
 
-  ok = gom_adapter_close_sync(self->adapter, &error);
+  ok = gom_adapter_close_sync (self->adapter, &error);
   if (!ok) {
-      if (error)
-        g_warning ("Error closing call record database `%s': %s",
-                   self->filename, error->message);
-      else
-        g_warning ("Unknown error closing call record database `%s'",
-                   self->filename);
-    }
+    if (error)
+      g_warning ("Error closing call record database `%s': %s",
+                 self->filename, error->message);
+    else
+      g_warning ("Unknown error closing call record database `%s'",
+                 self->filename);
+  }
 
   g_clear_object (&self->adapter);
 }
 
 
 static void
-open_repo_adapter_open_cb (GomAdapter *adapter,
-                           GAsyncResult *res,
+open_repo_adapter_open_cb (GomAdapter       *adapter,
+                           GAsyncResult     *res,
                            CallsRecordStore *self)
 {
   g_autoptr (GError) error = NULL;
@@ -351,12 +380,15 @@ open_repo_adapter_open_cb (GomAdapter *adapter,
                  self->filename);
 
     close_adapter (self);
-
+    g_signal_emit (self, signals[SIGNAL_DB_DONE], 0, FALSE);
   } else {
     g_debug ("Successfully opened call record database `%s'",
              self->filename);
     set_up_repo (self);
   }
+
+  self->busy--;
+  g_object_unref (self);
 }
 
 
@@ -382,20 +414,20 @@ open_repo (CallsRecordStore *self)
   uri = g_strdup_printf ("file:%s", self->filename);
   g_debug ("Opening call record database using URI `%s'", uri);
   self->adapter = gom_adapter_new ();
+  self->busy++;
   gom_adapter_open_async
     (self->adapter,
-     uri,
-     (GAsyncReadyCallback)open_repo_adapter_open_cb,
-     self);
+    uri,
+    (GAsyncReadyCallback) open_repo_adapter_open_cb,
+    g_object_ref (self));
 
   g_free (uri);
 }
 
 
-struct CallsRecordCallData
-{
+struct CallsRecordCallData {
   CallsRecordStore *self;
-  CallsUiCallData *call;
+  CallsUiCallData  *call;
 };
 
 
@@ -405,6 +437,7 @@ record_call_save_cb (GomResource                *resource,
                      struct CallsRecordCallData *data)
 {
   GObject * const call_obj = G_OBJECT (data->call);
+
   g_autoptr (GError) error = NULL;
   gboolean ok;
 
@@ -420,7 +453,7 @@ record_call_save_cb (GomResource                *resource,
 
   } else {
     g_debug ("Successfully saved new call record to database");
-    g_list_store_insert (G_LIST_STORE (data->self),
+    g_list_store_insert (G_LIST_STORE (data->self->list_store),
                          0,
                          CALLS_CALL_RECORD (resource));
 
@@ -463,13 +496,11 @@ record_call (CallsRecordStore *self,
                           record, g_object_unref);
 
   data = g_new (struct CallsRecordCallData, 1);
-  g_object_ref (self);
-  g_object_ref (call);
-  data->self = self;
-  data->call = call;
+  data->self = g_object_ref (self);
+  data->call = g_object_ref (call);
 
   gom_resource_save_async (GOM_RESOURCE (record),
-                           (GAsyncReadyCallback)record_call_save_cb,
+                           (GAsyncReadyCallback) record_call_save_cb,
                            data);
 }
 
@@ -497,14 +528,13 @@ update_cb (GomResource  *resource,
 
 
 static void
-stamp_call (CallsCallRecord  *record,
-            const gchar      *stamp_name)
+stamp_call (CallsCallRecord *record,
+            const char      *stamp_name)
 {
-  GObject *record_obj = G_OBJECT (record);
   GDateTime *stamp = NULL;
 
   /* Check the call has not already been stamped */
-  g_object_get (record_obj,
+  g_object_get (record,
                 stamp_name, &stamp,
                 NULL);
   if (stamp)
@@ -513,13 +543,13 @@ stamp_call (CallsCallRecord  *record,
 
   g_debug ("Stamping call `%s'", stamp_name);
   stamp = g_date_time_new_now_utc ();
-  g_object_set (record_obj,
+  g_object_set (record,
                 stamp_name, stamp,
                 NULL);
   g_date_time_unref (stamp);
 
   gom_resource_save_async (GOM_RESOURCE (record),
-                           (GAsyncReadyCallback)update_cb,
+                           (GAsyncReadyCallback) update_cb,
                            NULL);
 }
 
@@ -535,6 +565,9 @@ state_changed_cb (CallsRecordStore *self,
     g_object_get_data (call_obj, "calls-call-record");
   CallsCallRecordState new_rec_state, old_rec_state;
 
+  g_debug ("Call state changed from %s (%d) to %s (%d)",
+           cui_call_state_to_string (old_state), old_state,
+           cui_call_state_to_string (new_state), new_state);
 
   /* Check whether the call is recorded */
   if (!record) {
@@ -566,8 +599,7 @@ state_changed_cb (CallsRecordStore *self,
 
     case STARTED:
     default:
-      g_assert_not_reached ();
-      break;
+      goto critical_exit;
     }
     break;
 
@@ -580,16 +612,23 @@ state_changed_cb (CallsRecordStore *self,
     case STARTED:
     case ANSWERED:
     default:
-      g_assert_not_reached ();
-      break;
+      goto critical_exit;
     }
     break;
 
   case ENDED:
   default:
-    g_assert_not_reached ();
-    break;
+    goto critical_exit;
   }
+
+  return;
+
+critical_exit:
+  /* XXX Modem's may be buggy; let's print as much information as possible */
+  g_critical ("Unexpected state change occurred!\n"
+              "From %s (%s) to %s (%s)",
+              cui_call_state_to_string (old_state), record_state_to_string (old_rec_state),
+              cui_call_state_to_string (new_state), record_state_to_string (new_rec_state));
 }
 
 
@@ -603,7 +642,7 @@ call_added_cb (CallsRecordStore *self,
   g_assert (g_object_get_data (call_obj, "calls-call-start") == NULL);
   start = g_date_time_new_now_utc ();
   g_object_set_data_full (call_obj, "calls-call-start",
-                          start, (GDestroyNotify)g_date_time_unref);
+                          start, (GDestroyNotify) g_date_time_unref);
 
   if (!self->repository) {
     open_repo (self);
@@ -668,10 +707,7 @@ dispose (GObject *object)
 {
   CallsRecordStore *self = CALLS_RECORD_STORE (object);
 
-  g_list_store_remove_all (G_LIST_STORE (self));
-
   g_clear_object (&self->repository);
-  close_adapter (self);
 
   G_OBJECT_CLASS (calls_record_store_parent_class)->dispose (object);
 }
@@ -683,6 +719,7 @@ finalize (GObject *object)
   CallsRecordStore *self = CALLS_RECORD_STORE (object);
 
   g_free (self->filename);
+  close_adapter (self);
 
   G_OBJECT_CLASS (calls_record_store_parent_class)->finalize (object);
 }
@@ -696,26 +733,39 @@ calls_record_store_class_init (CallsRecordStoreClass *klass)
   object_class->constructed = constructed;
   object_class->dispose = dispose;
   object_class->finalize = finalize;
+
+  signals[SIGNAL_DB_DONE] = g_signal_new ("db-done",
+                                          G_TYPE_FROM_CLASS (klass),
+                                          G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+                                          NULL,
+                                          G_TYPE_NONE,
+                                          1,
+                                          G_TYPE_BOOLEAN);
 }
 
 
 static void
 calls_record_store_init (CallsRecordStore *self)
 {
-  gboolean exist_old, exist_new, new_is_dir;
-  g_autofree gchar *old_dir = g_build_filename (g_get_user_data_dir (),
-                                                "calls",
-                                                NULL);
-  g_autofree gchar *new_dir = g_build_filename (g_get_user_data_dir (),
-                                                APP_DATA_NAME,
-                                                NULL);
-  gchar *used_dir = NULL;
+  g_autofree char *old_dir = g_build_filename (g_get_user_data_dir (),
+                                               "calls",
+                                               NULL);
+  g_autofree char *new_dir = g_build_filename (g_get_user_data_dir (),
+                                               APP_DATA_NAME,
+                                               NULL);
+  const char *env_dir = g_getenv ("CALLS_RECORD_DIR");
+  char *used_dir = NULL;
+  gboolean exist_old;
+  gboolean exist_new;
+  gboolean new_is_dir;
 
   exist_old = g_file_test (old_dir, G_FILE_TEST_EXISTS);
   exist_new = g_file_test (new_dir, G_FILE_TEST_EXISTS);
   new_is_dir = g_file_test (new_dir, G_FILE_TEST_IS_DIR);
 
-  if (exist_old && !exist_new) {
+  if (env_dir) {
+    used_dir = (char *) env_dir;
+  } else if (exist_old && !exist_new) {
     g_debug ("Trying to move database from `%s' to `%s'", old_dir, new_dir);
 
     if (g_rename (old_dir, new_dir) == 0) {
@@ -725,23 +775,51 @@ calls_record_store_init (CallsRecordStore *self)
       g_debug ("Continuing to use old location");
       used_dir = old_dir;
     }
-  } else if (exist_new && new_is_dir)
+  } else if (exist_new && new_is_dir) {
     used_dir = new_dir;
-  else
+  } else {
     used_dir = old_dir;
+  }
 
   g_assert (used_dir);
 
   self->filename = g_build_filename (used_dir,
                                      RECORD_STORE_FILENAME,
                                      NULL);
+
+  self->list_store = g_list_store_new (CALLS_TYPE_CALL_RECORD);
 }
 
 
 CallsRecordStore *
 calls_record_store_new (void)
 {
-  return g_object_new (CALLS_TYPE_RECORD_STORE,
-                       "item-type", CALLS_TYPE_CALL_RECORD,
-                       NULL);
+  return g_object_new (CALLS_TYPE_RECORD_STORE, NULL);
+}
+
+
+GListModel *
+calls_record_store_get_list_model (CallsRecordStore *self)
+{
+  return G_LIST_MODEL (self->list_store);
+}
+
+/**
+ * calls_record_store_is_busy:
+ * @self: The record store
+ *
+ * Check whether there are async database operations in flight. It
+ * is only save to dispose the record store when this function returns
+ * `FALSE`.
+ *
+ * Returns: `TRUE` when there are async db operations, otherwise
+ *   `FALSE`.
+ */
+gboolean
+calls_record_store_is_busy (CallsRecordStore *self)
+{
+  g_assert (CALLS_IS_RECORD_STORE (self));
+  g_assert (self->busy >= 0);
+
+  return !!self->busy;
 }
